@@ -17,7 +17,6 @@
 #include <QCoreApplication>
 #include <QTimer>
 
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <utime.h>
 
@@ -151,6 +150,12 @@ enum MTPSlave::Url MTPSlave::checkUrl(const QUrl &url)
     }
     if (url.path().startsWith(QLatin1Char('/'))) {
         return Url::Valid;
+    } else if (url.scheme() == QLatin1String("mtp") && url.path().isEmpty()) {
+        QUrl newUrl = url;
+        newUrl.setPath(QLatin1String("/"));
+        redirection(newUrl);
+
+        return Url::Redirected;
     }
     return Url::Invalid;
 }
@@ -185,14 +190,14 @@ void MTPSlave::listDir(const QUrl &url)
     if (pathItems.isEmpty()) {
         qCDebug(LOG_KIO_MTP) << "Root directory, listing devices";
 
-        totalSize(filesize_t(m_kmtpDaemon.devices().size()));
-
         const auto devices = m_kmtpDaemon.devices();
+        totalSize(filesize_t(devices.size()));
+
         for (const KMTPDeviceInterface *device : devices) {
             listEntry(getEntry(device));
         }
 
-        qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Devices:" << m_kmtpDaemon.devices().size();
+        qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Devices:" << devices.size();
         finished();
         return;
     }
@@ -215,7 +220,7 @@ void MTPSlave::listDir(const QUrl &url)
                 finished();
                 qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Storage media:" << storages.count();
             } else {
-                error(ERR_SLAVE_DEFINED, i18nc("Message shown when attempting to access an MTP device that is not fully accessible yet", "Could not access device. Make sure it is unlocked, and tap \"Allow\" on the popup on its screen. If that does not work, make sure MTP is enabled in its USB connection settings."));
+                error(ERR_WORKER_DEFINED, i18nc("Message shown when attempting to access an MTP device that is not fully accessible yet", "Could not access device. Make sure it is unlocked, and tap \"Allow\" on the popup on its screen. If that does not work, make sure MTP is enabled in its USB connection settings."));
             }
         } else {
             // list files and folders
